# $Id$

package POE::Component::IKC::Edge;

use strict;
use POE::Component::IKC::Responder;

###########################################################################
# Responder session in the edge node

use vars qw( @ISA );
@ISA = qw(POE::Component::IKC::Responder );

sub create_responder_obj
{
    my $package = shift;
    return POE::Component::IKC::Edge::Object->new( @_ );
}


###########################################################################
# Responder object in an edge node
package POE::Component::IKC::Edge::Object;

use strict;

use POE;
use Data::Denter;

use POE::Component::IKC::Specifier;

use vars qw( @ISA );
@ISA = qw( POE::Component::IKC::Responder::Object );

sub DEBUG () { 0 }

sub request
{
    my($self, $request)=@_;

    my $to=specifier_parse($request->{event});

    DEBUG and 
            warn "Request to ", Denter $to;

    unless( $to->{kernel} eq 'IKC-edge' and $to->{session} eq 'IKC-edge' ) {
        return $self->SUPER::request( $request );
    }

    eval {
        if( $to->{state} eq 'inform_edge_monitor' ) {
            $self->inform_edge_monitor( $request->{params} );
        }
        else {
            die "Unknown state $to->{state} for IKC-edge";
        }
    };
    warn "$to->{state}: $@" if $@;
}


# Get a list of channels that correspond to a kernel
sub channel_list
{
    my( $self, $name ) = @_;
    my @channels = $self->SUPER::channel_list( $name );
    return @channels if @channels;

    DEBUG and warn "YOW!  I want my proxy-node for $name!";

    my $channel = $self->node_channel;
    return $channel if $channel;

    warn "Not connected to proxy-node";
    return;
}

# Get the channel that's the proxy-node
sub node_channel
{
    my( $self, $name ) = @_;

    my $known = $self->{channel};

    foreach my $chan ( keys %$known ) {
        DEBUG and warn "Is $chan ($known->{$chan}) $self->{local_channel}";
        return $known->{$chan} if $known->{$chan} 
                                            ne $self->{local_channel};
    }

    return;
}


# If this edge sets a new monitor, we want to inform the proxy-node
sub monitor
{
    my( $self, $sender, $name, $states ) = @_;

    my $channel = $self->node_channel();
    if( $channel ) {
        if( $states ) {
            my $proxy_interest;

            # We want to tell the proxy-node that we are insterested in $name
            foreach my $what ( keys %$states ) {
                next unless $what eq 'register' or $what eq 'unregister';
                $proxy_interest->{ $name } = $what;
            }

            if( $proxy_interest ) {
                # now tell the proxy 
                $self->to_node( 'node_monitor' => $proxy_interest );
            }
        }
        else {  # no longer interested

            warn "self = $self";
            $self->to_node( 'node_unmonitor', { delete=>$name } );
        }
    }

    # tell local IKC about it all
    $self->SUPER::monitor( $sender, $name, $states );
}

# If we create a new connection (assume it's a proxy-node) 
# we want to inform it of what we are currently monitoring
sub register 
{
    my($self, $channel, $rid, $aliases)=@_;

    $self->SUPER::register( $channel, $rid, $aliases );

    # go through the list of monitors and inform the other end 
    # of the ones we are interested in
    my $proxy_interest;

    DEBUG and 
        warn "monitors ", Denter $self->{monitors};
    while( my( $n, $ms ) = each %{ $self->{monitors} } ) {
        next if $n eq '*';
        foreach my $sender (keys %$ms) {
            my $states=$ms->{$sender};
            
            foreach my $event ( keys %$states ) {
                next unless $event eq 'register' or $event eq 'unregister';
                $proxy_interest->{ $n } = $event;
            }
        }
    }    
    return unless $proxy_interest;

    $proxy_interest->{__on_register} = 1;

    # now tell the proxy 
    $self->to_node( 'node_monitor' => $proxy_interest );
    return;
}


# The proxy-node wants us to know a kernel has unregistered
sub inform_edge_monitor
{
    my( $self, $param ) = @_;

    DEBUG and warn "Proxy $param->{rid} $param->{event}";
    $self->inform_monitors( $param->{rid}, $param->{event} );
}


# Send a request to the proxy-node
sub to_node
{
    my( $self, $event, $params ) = @_;

    my $channel = $self->node_channel();
    return unless $channel;

    DEBUG and warn "Sending a request to IKC-node via channel $channel";

    # This must be a post(), because we could be called from register(),
    # and register is now called before the filter is switched
    $poe_kernel->post( $channel => 'send', 
                        { event => { kernel=>'IKC-node',
                                     session=>'IKC-node',
                                     state=>$event },

                          from => { kernel=>$poe_kernel->ID, 
                                    session=>'IKC-edge' },
                          params => $params 
                        }
                      );
}


1;

__END__

$Log$

=head1 NAME

POE::Component::IKC::Edge - IKC responder for edge nodes

=head1 SYNOPSIS

    # Using
    use POE::Component::IKC::Edge;
    POE::Component::IKC::Edge->new;
    

    # Overloading
    package My::Node;
    use POE::Component::IKC::Edge;

    @ISA = qw( POE::Component::IKC::Edge );

    sub create_responder_obj
    {
        my $package = shift;
    
        My::Responder->new( @_ );
    }

=head1 DESCRIPTION





=head1 METHODS

=head2 spawn()

Create the IKC responder for an edge node.

=head2 create_responder_obj


=head1 POE::Component::IKC::Edge::Object

This class sub-classes L<POE::Component::IKC::Responder::Object>.  It
adds a few features needed for edge nodes.

First of all, it must forward all request to unknown kernels to the proxy
node.  Secondly, the proxy node needs know about all the kernels we are
monitoring, so it can tell the edge node about relevant events.  The proxy
node only knows that an edge node is monitoring a kernel.  It does not know
which session is monitoring a given kernel.

This implementation expects there to be only one IKC connection active; the
connection to the proxy node.  This is probably the only connection you will
need anyway.

=head2 $self->request( $request )

Look at the incoming IKC request to see if it's a message from the proxy
node.  If not, default behaviour.


=head2 $self->channel_list( $name )

Looks for a list of channels for the kernel $name.  If none are found, 
returns the proxy node's channel, as found with L<node_channel>.

=head2 $self->node_channel()

Returns the channel that talks to the proxy node.  Currently, it expects
that the first non-local channel to be the proxy.

=head2 $self->monitor( $sender, $name, $states )

If a local session wants to monitor a register or unregister, tell 
the proxy node that this edge node is interest in that kernel.

C<$sender> is the session that is doing the monitoring.
C<$name> is the kernel to be monitored.
C<$states> is what the session wants to monitor.

=head2 $self->register( $channel, $rid, $aliases )

Called when edge node connects to the proxy node.  Responds by telling the
proxy node about all the kernels this node is monitoring.

=head2 $self->inform_edge_monitor( $param );

Message from the proxy node responder, indicating that something happened to
a kernel the edge node was monitoring.

C<$param> is a hashref with 2 keys: {rid}, the name of the kernel
and {event}, what happened to it.

C<inform_edge_monitor> passes these on to C<inform_monitor>.


=head2 $self->to_node( $event, $params )

Send a message to the proxy node's responder.  

The proxy node's kernel is called 'IKC-node', the session is 'IKC-node'.

=head1 BUGS

Many things are un-implemented or untested.

Changing the responder object is overly complex.

=head1 AUTHOR

Philip Gwyn, <perl-ikc at pied.nu>

=head1 SEE ALSO

L<POE>, L<POE::Component::IKC::Node>, L<POE::Component::IKC::Connection>,
L<POE::Component::IKC::Responder>


=cut
